@@ -1,6 +1,8 @@
-# Copyright (c) 2006-2011 Mitch Garnaat http://garnaat.org/
+# Copyright (c) 2006-2012 Mitch Garnaat http://garnaat.org/
 # Copyright (c) 2010-2011, Eucalyptus Systems, Inc.
 # Copyright (c) 2011, Nexenta Systems Inc.
+# Copyright (c) 2012 Amazon.com, Inc. or its affiliates.
+# Copyright (c) 2010, Google, Inc.
 # All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
@@ -25,18 +27,30 @@
 from boto.pyami.config import Config, BotoConfigLocations
 from boto.storage_uri import BucketStorageUri, FileStorageUri
 import boto.plugin
-import os, re, sys
+import os
+import platform
+import re
+import sys
 import logging
 import logging.config
 import urlparse
 from boto.exception import InvalidUriError
 
-__version__ = '2.2.2'
-Version = __version__ # for backware compatibility
+__version__ = '2.9.6'
+Version = __version__  # for backware compatibility
 
 UserAgent = 'Boto/%s (%s)' % (__version__, sys.platform)
 config = Config()
 
+# Regex to disallow buckets violating charset or not [3..255] chars total.
+BUCKET_NAME_RE = re.compile(r'^[a-zA-Z0-9][a-zA-Z0-9\._-]{1,253}[a-zA-Z0-9]$')
+# Regex to disallow buckets with individual DNS labels longer than 63.
+TOO_LONG_DNS_NAME_COMP = re.compile(r'[-_a-z0-9]{64}')
+GENERATION_RE = re.compile(r'(?P<versionless_uri_str>.+)'
+                           r'#(?P<generation>[0-9]+)$')
+VERSION_RE = re.compile('(?P<versionless_uri_str>.+)#(?P<version_id>.+)$')
+
+
 def init_logging():
     for file in BotoConfigLocations:
         try:
@@ -44,15 +58,20 @@ def init_logging():
         except:
             pass
 
+
 class NullHandler(logging.Handler):
     def emit(self, record):
         pass
 
 log = logging.getLogger('boto')
+perflog = logging.getLogger('boto.perf')
 log.addHandler(NullHandler())
+perflog.addHandler(NullHandler())
 init_logging()
 
 # convenience function to set logging to a particular file
+
+
 def set_file_logger(name, filepath, level=logging.INFO, format_string=None):
     global log
     if not format_string:
@@ -66,6 +85,7 @@ def set_file_logger(name, filepath, level=logging.INFO, format_string=None):
     logger.addHandler(fh)
     log = logger
 
+
 def set_stream_logger(name, level=logging.DEBUG, format_string=None):
     global log
     if not format_string:
@@ -79,6 +99,7 @@ def set_stream_logger(name, level=logging.DEBUG, format_string=None):
     logger.addHandler(fh)
     log = logger
 
+
 def connect_sqs(aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
     """
     :type aws_access_key_id: string
@@ -93,6 +114,7 @@ def connect_sqs(aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
     from boto.sqs.connection import SQSConnection
     return SQSConnection(aws_access_key_id, aws_secret_access_key, **kwargs)
 
+
 def connect_s3(aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
     """
     :type aws_access_key_id: string
@@ -107,6 +129,7 @@ def connect_s3(aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
     from boto.s3.connection import S3Connection
     return S3Connection(aws_access_key_id, aws_secret_access_key, **kwargs)
 
+
 def connect_gs(gs_access_key_id=None, gs_secret_access_key=None, **kwargs):
     """
     @type gs_access_key_id: string
@@ -121,6 +144,7 @@ def connect_gs(gs_access_key_id=None, gs_secret_access_key=None, **kwargs):
     from boto.gs.connection import GSConnection
     return GSConnection(gs_access_key_id, gs_secret_access_key, **kwargs)
 
+
 def connect_ec2(aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
     """
     :type aws_access_key_id: string
@@ -135,6 +159,7 @@ def connect_ec2(aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
     from boto.ec2.connection import EC2Connection
     return EC2Connection(aws_access_key_id, aws_secret_access_key, **kwargs)
 
+
 def connect_elb(aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
     """
     :type aws_access_key_id: string
@@ -149,7 +174,9 @@ def connect_elb(aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
     from boto.ec2.elb import ELBConnection
     return ELBConnection(aws_access_key_id, aws_secret_access_key, **kwargs)
 
-def connect_autoscale(aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
+
+def connect_autoscale(aws_access_key_id=None, aws_secret_access_key=None,
+                      **kwargs):
     """
     :type aws_access_key_id: string
     :param aws_access_key_id: Your AWS Access Key ID
@@ -161,9 +188,12 @@ def connect_autoscale(aws_access_key_id=None, aws_secret_access_key=None, **kwar
     :return: A connection to Amazon's Auto Scaling Service
     """
     from boto.ec2.autoscale import AutoScaleConnection
-    return AutoScaleConnection(aws_access_key_id, aws_secret_access_key, **kwargs)
+    return AutoScaleConnection(aws_access_key_id, aws_secret_access_key,
+                               **kwargs)
 
-def connect_cloudwatch(aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
+
+def connect_cloudwatch(aws_access_key_id=None, aws_secret_access_key=None,
+                       **kwargs):
     """
     :type aws_access_key_id: string
     :param aws_access_key_id: Your AWS Access Key ID
@@ -175,7 +205,9 @@ def connect_cloudwatch(aws_access_key_id=None, aws_secret_access_key=None, **kwa
     :return: A connection to Amazon's EC2 Monitoring service
     """
     from boto.ec2.cloudwatch import CloudWatchConnection
-    return CloudWatchConnection(aws_access_key_id, aws_secret_access_key, **kwargs)
+    return CloudWatchConnection(aws_access_key_id, aws_secret_access_key,
+                                **kwargs)
+
 
 def connect_sdb(aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
     """
@@ -191,6 +223,7 @@ def connect_sdb(aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
     from boto.sdb.connection import SDBConnection
     return SDBConnection(aws_access_key_id, aws_secret_access_key, **kwargs)
 
+
 def connect_fps(aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
     """
     :type aws_access_key_id: string
@@ -205,7 +238,9 @@ def connect_fps(aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
     from boto.fps.connection import FPSConnection
     return FPSConnection(aws_access_key_id, aws_secret_access_key, **kwargs)
 
-def connect_mturk(aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
+
+def connect_mturk(aws_access_key_id=None, aws_secret_access_key=None,
+                  **kwargs):
     """
     :type aws_access_key_id: string
     :param aws_access_key_id: Your AWS Access Key ID
@@ -219,7 +254,9 @@ def connect_mturk(aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
     from boto.mturk.connection import MTurkConnection
     return MTurkConnection(aws_access_key_id, aws_secret_access_key, **kwargs)
 
-def connect_cloudfront(aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
+
+def connect_cloudfront(aws_access_key_id=None, aws_secret_access_key=None,
+                       **kwargs):
     """
     :type aws_access_key_id: string
     :param aws_access_key_id: Your AWS Access Key ID
@@ -231,7 +268,9 @@ def connect_cloudfront(aws_access_key_id=None, aws_secret_access_key=None, **kwa
     :return: A connection to FPS
     """
     from boto.cloudfront import CloudFrontConnection
-    return CloudFrontConnection(aws_access_key_id, aws_secret_access_key, **kwargs)
+    return CloudFrontConnection(aws_access_key_id, aws_secret_access_key,
+                                **kwargs)
+
 
 def connect_vpc(aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
     """
@@ -247,6 +286,7 @@ def connect_vpc(aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
     from boto.vpc import VPCConnection
     return VPCConnection(aws_access_key_id, aws_secret_access_key, **kwargs)
 
+
 def connect_rds(aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
     """
     :type aws_access_key_id: string
@@ -261,6 +301,7 @@ def connect_rds(aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
     from boto.rds import RDSConnection
     return RDSConnection(aws_access_key_id, aws_secret_access_key, **kwargs)
 
+
 def connect_emr(aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
     """
     :type aws_access_key_id: string
@@ -275,6 +316,7 @@ def connect_emr(aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
     from boto.emr import EmrConnection
     return EmrConnection(aws_access_key_id, aws_secret_access_key, **kwargs)
 
+
 def connect_sns(aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
     """
     :type aws_access_key_id: string
@@ -304,7 +346,9 @@ def connect_iam(aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
     from boto.iam import IAMConnection
     return IAMConnection(aws_access_key_id, aws_secret_access_key, **kwargs)
 
-def connect_route53(aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
+
+def connect_route53(aws_access_key_id=None, aws_secret_access_key=None,
+                    **kwargs):
     """
     :type aws_access_key_id: string
     :param aws_access_key_id: Your AWS Access Key ID
@@ -316,9 +360,12 @@ def connect_route53(aws_access_key_id=None, aws_secret_access_key=None, **kwargs
     :return: A connection to Amazon's Route53 DNS Service
     """
     from boto.route53 import Route53Connection
-    return Route53Connection(aws_access_key_id, aws_secret_access_key, **kwargs)
+    return Route53Connection(aws_access_key_id, aws_secret_access_key,
+                             **kwargs)
 
-def connect_cloudformation(aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
+
+def connect_cloudformation(aws_access_key_id=None, aws_secret_access_key=None,
+                           **kwargs):
     """
     :type aws_access_key_id: string
     :param aws_access_key_id: Your AWS Access Key ID
@@ -330,7 +377,9 @@ def connect_cloudformation(aws_access_key_id=None, aws_secret_access_key=None, *
     :return: A connection to Amazon's CloudFormation Service
     """
     from boto.cloudformation import CloudFormationConnection
-    return CloudFormationConnection(aws_access_key_id, aws_secret_access_key, **kwargs)
+    return CloudFormationConnection(aws_access_key_id, aws_secret_access_key,
+                                    **kwargs)
+
 
 def connect_euca(host=None, aws_access_key_id=None, aws_secret_access_key=None,
                  port=8773, path='/services/Eucalyptus', is_secure=False,
@@ -370,7 +419,26 @@ def connect_euca(host=None, aws_access_key_id=None, aws_secret_access_key=None,
                          region=reg, port=port, path=path,
                          is_secure=is_secure, **kwargs)
 
-def connect_ec2_endpoint(url, aws_access_key_id=None, aws_secret_access_key=None, 
+
+def connect_glacier(aws_access_key_id=None, aws_secret_access_key=None,
+                    **kwargs):
+    """
+    :type aws_access_key_id: string
+    :param aws_access_key_id: Your AWS Access Key ID
+
+    :type aws_secret_access_key: string
+    :param aws_secret_access_key: Your AWS Secret Access Key
+
+    :rtype: :class:`boto.glacier.layer2.Layer2`
+    :return: A connection to Amazon's Glacier Service
+    """
+    from boto.glacier.layer2 import Layer2
+    return Layer2(aws_access_key_id, aws_secret_access_key,
+                  **kwargs)
+
+
+def connect_ec2_endpoint(url, aws_access_key_id=None,
+                         aws_secret_access_key=None,
                          **kwargs):
     """
     Connect to an EC2 Api endpoint.  Additional arguments are passed
@@ -397,14 +465,16 @@ def connect_ec2_endpoint(url, aws_access_key_id=None, aws_secret_access_key=None
     if not 'is_secure' in kwargs:
         kwargs['is_secure'] = (purl.scheme == "https")
 
-    kwargs['region'] = RegionInfo(name = purl.hostname,
-        endpoint = purl.hostname)
-    kwargs['aws_access_key_id']=aws_access_key_id
-    kwargs['aws_secret_access_key']=aws_secret_access_key
+    kwargs['region'] = RegionInfo(name=purl.hostname,
+                                  endpoint=purl.hostname)
+    kwargs['aws_access_key_id'] = aws_access_key_id
+    kwargs['aws_secret_access_key'] = aws_secret_access_key
 
     return(connect_ec2(**kwargs))
 
-def connect_walrus(host=None, aws_access_key_id=None, aws_secret_access_key=None,
+
+def connect_walrus(host=None, aws_access_key_id=None,
+                   aws_secret_access_key=None,
                    port=8773, path='/services/Walrus', is_secure=False,
                    **kwargs):
     """
@@ -436,12 +506,13 @@ def connect_walrus(host=None, aws_access_key_id=None, aws_secret_access_key=None
                                            None)
     if not host:
         host = config.get('Boto', 'walrus_host', None)
-        
+
     return S3Connection(aws_access_key_id, aws_secret_access_key,
                         host=host, port=port, path=path,
                         calling_format=OrdinaryCallingFormat(),
                         is_secure=is_secure, **kwargs)
 
+
 def connect_ses(aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
     """
     :type aws_access_key_id: string
@@ -456,6 +527,7 @@ def connect_ses(aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
     from boto.ses import SESConnection
     return SESConnection(aws_access_key_id, aws_secret_access_key, **kwargs)
 
+
 def connect_sts(aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
     """
     :type aws_access_key_id: string
@@ -470,21 +542,21 @@ def connect_sts(aws_access_key_id=None, aws_secret_access_key=None, **kwargs):
     from boto.sts import STSConnection
     return STSConnection(aws_access_key_id, aws_secret_access_key, **kwargs)
 
+
 def connect_ia(ia_access_key_id=None, ia_secret_access_key=None,
                is_secure=False, **kwargs):
     """
     Connect to the Internet Archive via their S3-like API.
 
     :type ia_access_key_id: string
-    :param ia_access_key_id: Your IA Access Key ID.  This will also look in your
-                             boto config file for an entry in the Credentials
-                             section called "ia_access_key_id"
+    :param ia_access_key_id: Your IA Access Key ID.  This will also look
+        in your boto config file for an entry in the Credentials
+        section called "ia_access_key_id"
 
     :type ia_secret_access_key: string
     :param ia_secret_access_key: Your IA Secret Access Key.  This will also
-                                 look in your boto config file for an entry
-                                 in the Credentials section called
-                                 "ia_secret_access_key"
+        look in your boto config file for an entry in the Credentials
+        section called "ia_secret_access_key"
 
     :rtype: :class:`boto.s3.connection.S3Connection`
     :return: A connection to the Internet Archive
@@ -502,6 +574,7 @@ def connect_ia(ia_access_key_id=None, ia_secret_access_key=None,
                         calling_format=OrdinaryCallingFormat(),
                         is_secure=is_secure, **kwargs)
 
+
 def connect_dynamodb(aws_access_key_id=None,
                      aws_secret_access_key=None,
                      **kwargs):
@@ -518,46 +591,134 @@ def connect_dynamodb(aws_access_key_id=None,
     from boto.dynamodb.layer2 import Layer2
     return Layer2(aws_access_key_id, aws_secret_access_key, **kwargs)
 
-def check_extensions(module_name, module_path):
+
+def connect_swf(aws_access_key_id=None,
+                aws_secret_access_key=None,
+                **kwargs):
     """
-    This function checks for extensions to boto modules.  It should be called in the
-    __init__.py file of all boto modules.  See:
-    http://code.google.com/p/boto/wiki/ExtendModules
+    :type aws_access_key_id: string
+    :param aws_access_key_id: Your AWS Access Key ID
 
-    for details.
+    :type aws_secret_access_key: string
+    :param aws_secret_access_key: Your AWS Secret Access Key
+
+    :rtype: :class:`boto.swf.layer1.Layer1`
+    :return: A connection to the Layer1 interface for SWF.
     """
-    option_name = '%s_extend' % module_name
-    version = config.get('Boto', option_name, None)
-    if version:
-        dirname = module_path[0]
-        path = os.path.join(dirname, version)
-        if os.path.isdir(path):
-            log.info('extending module %s with: %s' % (module_name, path))
-            module_path.insert(0, path)
+    from boto.swf.layer1 import Layer1
+    return Layer1(aws_access_key_id, aws_secret_access_key, **kwargs)
 
-_aws_cache = {}
 
-def _get_aws_conn(service):
-    global _aws_cache
-    conn = _aws_cache.get(service)
-    if not conn:
-        meth = getattr(sys.modules[__name__], 'connect_' + service)
-        conn = meth()
-        _aws_cache[service] = conn
-    return conn
+def connect_cloudsearch(aws_access_key_id=None,
+                        aws_secret_access_key=None,
+                        **kwargs):
+    """
+    :type aws_access_key_id: string
+    :param aws_access_key_id: Your AWS Access Key ID
+
+    :type aws_secret_access_key: string
+    :param aws_secret_access_key: Your AWS Secret Access Key
+
+    :rtype: :class:`boto.ec2.autoscale.CloudSearchConnection`
+    :return: A connection to Amazon's CloudSearch service
+    """
+    from boto.cloudsearch.layer2 import Layer2
+    return Layer2(aws_access_key_id, aws_secret_access_key,
+                  **kwargs)
+
+
+def connect_beanstalk(aws_access_key_id=None,
+                      aws_secret_access_key=None,
+                      **kwargs):
+    """
+    :type aws_access_key_id: string
+    :param aws_access_key_id: Your AWS Access Key ID
+
+    :type aws_secret_access_key: string
+    :param aws_secret_access_key: Your AWS Secret Access Key
+
+    :rtype: :class:`boto.beanstalk.layer1.Layer1`
+    :return: A connection to Amazon's Elastic Beanstalk service
+    """
+    from boto.beanstalk.layer1 import Layer1
+    return Layer1(aws_access_key_id, aws_secret_access_key, **kwargs)
+
+
+def connect_elastictranscoder(aws_access_key_id=None,
+                              aws_secret_access_key=None,
+                              **kwargs):
+    """
+    :type aws_access_key_id: string
+    :param aws_access_key_id: Your AWS Access Key ID
+
+    :type aws_secret_access_key: string
+    :param aws_secret_access_key: Your AWS Secret Access Key
+
+    :rtype: :class:`boto.ets.layer1.ElasticTranscoderConnection`
+    :return: A connection to Amazon's Elastic Transcoder service
+    """
+    from boto.elastictranscoder.layer1 import ElasticTranscoderConnection
+    return ElasticTranscoderConnection(
+        aws_access_key_id=aws_access_key_id,
+        aws_secret_access_key=aws_secret_access_key,
+        **kwargs)
+
+
+def connect_opsworks(aws_access_key_id=None,
+                     aws_secret_access_key=None,
+                     **kwargs):
+    from boto.opsworks.layer1 import OpsWorksConnection
+    return OpsWorksConnection(
+        aws_access_key_id=aws_access_key_id,
+        aws_secret_access_key=aws_secret_access_key,
+        **kwargs)
+
+
+def connect_redshift(aws_access_key_id=None,
+                     aws_secret_access_key=None,
+                     **kwargs):
+    """
+    :type aws_access_key_id: string
+    :param aws_access_key_id: Your AWS Access Key ID
+
+    :type aws_secret_access_key: string
+    :param aws_secret_access_key: Your AWS Secret Access Key
+
+    :rtype: :class:`boto.redshift.layer1.RedshiftConnection`
+    :return: A connection to Amazon's Redshift service
+    """
+    from boto.redshift.layer1 import RedshiftConnection
+    return RedshiftConnection(
+        aws_access_key_id=aws_access_key_id,
+        aws_secret_access_key=aws_secret_access_key,
+        **kwargs
+    )
+
+
+def connect_support(aws_access_key_id=None,
+                    aws_secret_access_key=None,
+                    **kwargs):
+    """
+    :type aws_access_key_id: string
+    :param aws_access_key_id: Your AWS Access Key ID
+
+    :type aws_secret_access_key: string
+    :param aws_secret_access_key: Your AWS Secret Access Key
+
+    :rtype: :class:`boto.support.layer1.SupportConnection`
+    :return: A connection to Amazon's Support service
+    """
+    from boto.support.layer1 import SupportConnection
+    return SupportConnection(
+        aws_access_key_id=aws_access_key_id,
+        aws_secret_access_key=aws_secret_access_key,
+        **kwargs
+    )
 
-def lookup(service, name):
-    global _aws_cache
-    conn = _get_aws_conn(service)
-    obj = _aws_cache.get('.'.join((service, name)), None)
-    if not obj:
-        obj = conn.lookup(name)
-        _aws_cache['.'.join((service, name))] = obj
-    return obj
 
 def storage_uri(uri_str, default_scheme='file', debug=0, validate=True,
                 bucket_storage_uri_class=BucketStorageUri,
-                suppress_consec_slashes=True):
+                suppress_consec_slashes=True, is_latest=False):
     """
     Instantiate a StorageUri from a URI string.
 
@@ -573,6 +734,9 @@ def storage_uri(uri_str, default_scheme='file', debug=0, validate=True,
     :param bucket_storage_uri_class: Allows mocking for unit tests.
     :param suppress_consec_slashes: If provided, controls whether
         consecutive slashes will be suppressed in key paths.
+    :type is_latest: bool
+    :param is_latest: whether this versioned object represents the
+        current version.
 
     We allow validate to be disabled to allow caller
     to implement bucket-level wildcarding (outside the boto library;
@@ -584,13 +748,17 @@ def storage_uri(uri_str, default_scheme='file', debug=0, validate=True,
     ``uri_str`` must be one of the following formats:
 
     * gs://bucket/name
+    * gs://bucket/name#ver
     * s3://bucket/name
     * gs://bucket
     * s3://bucket
-    * filename
+    * filename (which could be a Unix path like /a/b/c or a Windows path like
+      C:\a\b\c)
 
-    The last example uses the default scheme ('file', unless overridden)
+    The last example uses the default scheme ('file', unless overridden).
     """
+    version_id = None
+    generation = None
 
     # Manually parse URI components instead of using urlparse.urlparse because
     # what we're calling URIs don't really fit the standard syntax for URIs
@@ -600,8 +768,15 @@ def storage_uri(uri_str, default_scheme='file', debug=0, validate=True,
         # Check for common error: user specifies gs:bucket instead
         # of gs://bucket. Some URI parsers allow this, but it can cause
         # confusion for callers, so we don't.
-        if uri_str.find(':') != -1:
-            raise InvalidUriError('"%s" contains ":" instead of "://"' % uri_str)
+        colon_pos = uri_str.find(':')
+        if colon_pos != -1:
+            # Allow Windows path names including drive letter (C: etc.)
+            drive_char = uri_str[0].lower()
+            if not (platform.system().lower().startswith('windows')
+                    and colon_pos == 1
+                    and drive_char >= 'a' and drive_char <= 'z'):
+              raise InvalidUriError('"%s" contains ":" instead of "://"' %
+                                    uri_str)
         scheme = default_scheme.lower()
         path = uri_str
     else:
@@ -620,23 +795,39 @@ def storage_uri(uri_str, default_scheme='file', debug=0, validate=True,
     else:
         path_parts = path.split('/', 1)
         bucket_name = path_parts[0]
-        if (validate and bucket_name and
-            # Disallow buckets violating charset or not [3..255] chars total.
-            (not re.match('^[a-z0-9][a-z0-9\._-]{1,253}[a-z0-9]$', bucket_name)
-            # Disallow buckets with individual DNS labels longer than 63.
-             or re.search('[-_a-z0-9]{64}', bucket_name))):
-            raise InvalidUriError('Invalid bucket name in URI "%s"' % uri_str)
-        # If enabled, ensure the bucket name is valid, to avoid possibly
-        # confusing other parts of the code. (For example if we didn't
+        object_name = ''
+        # If validate enabled, ensure the bucket name is valid, to avoid
+        # possibly confusing other parts of the code. (For example if we didn't
         # catch bucket names containing ':', when a user tried to connect to
         # the server with that name they might get a confusing error about
         # non-integer port numbers.)
-        object_name = ''
+        if (validate and bucket_name and
+            (not BUCKET_NAME_RE.match(bucket_name)
+             or TOO_LONG_DNS_NAME_COMP.search(bucket_name))):
+            raise InvalidUriError('Invalid bucket name in URI "%s"' % uri_str)
+        if scheme == 'gs':
+            match = GENERATION_RE.search(path)
+            if match:
+                md = match.groupdict()
+                versionless_uri_str = md['versionless_uri_str']
+                path_parts = versionless_uri_str.split('/', 1)
+                generation = int(md['generation'])
+        elif scheme == 's3':
+            match = VERSION_RE.search(path)
+            if match:
+                md = match.groupdict()
+                versionless_uri_str = md['versionless_uri_str']
+                path_parts = versionless_uri_str.split('/', 1)
+                version_id = md['version_id']
+        else:
+            raise InvalidUriError('Unrecognized scheme "%s"' % scheme)
         if len(path_parts) > 1:
             object_name = path_parts[1]
         return bucket_storage_uri_class(
             scheme, bucket_name, object_name, debug,
-            suppress_consec_slashes=suppress_consec_slashes)
+            suppress_consec_slashes=suppress_consec_slashes,
+            version_id=version_id, generation=generation, is_latest=is_latest)
+
 
 def storage_uri_for_key(key):
     """Returns a StorageUri for the given key.
