@@ -1,4 +1,5 @@
 # Copyright (c) 2009 Reza Lotun http://reza.lotun.name/
+# Copyright (c) 2012 Amazon.com, Inc. or its affiliates.  All Rights Reserved
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the
@@ -19,13 +20,15 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 # IN THE SOFTWARE.
 
-
 from datetime import datetime
-import base64
 from boto.resultset import ResultSet
 from boto.ec2.elb.listelement import ListElement
+import boto.utils
+import base64
 
 # this should use the corresponding object from boto.ec2
+
+
 class Ebs(object):
     def __init__(self, connection=None, snapshot_id=None, volume_size=None):
         self.connection = connection
@@ -70,7 +73,8 @@ class BlockDeviceMapping(object):
         self.ebs = None
 
     def __repr__(self):
-        return 'BlockDeviceMapping(%s, %s)' % (self.device_name, self.virtual_name)
+        return 'BlockDeviceMapping(%s, %s)' % (self.device_name,
+                                               self.virtual_name)
 
     def startElement(self, name, attrs, connection):
         if name == 'Ebs':
@@ -89,7 +93,8 @@ class LaunchConfiguration(object):
                  key_name=None, security_groups=None, user_data=None,
                  instance_type='m1.small', kernel_id=None,
                  ramdisk_id=None, block_device_mappings=None,
-                 instance_monitoring=False):
+                 instance_monitoring=False, spot_price=None,
+                 instance_profile_name=None, ebs_optimized=False):
         """
         A launch configuration.
 
@@ -98,14 +103,14 @@ class LaunchConfiguration(object):
 
         :type image_id: str
         :param image_id: Unique ID of the Amazon Machine Image (AMI) which was
-                         assigned during registration.
+            assigned during registration.
 
         :type key_name: str
         :param key_name: The name of the EC2 key pair.
 
         :type security_groups: list
         :param security_groups: Names of the security groups with which to
-                                associate the EC2 instances.
+            associate the EC2 instances.
 
         :type user_data: str
         :param user_data: The user data available to launched EC2 instances.
@@ -121,11 +126,24 @@ class LaunchConfiguration(object):
 
         :type block_device_mappings: list
         :param block_device_mappings: Specifies how block devices are exposed
-                                      for instances
+            for instances
 
         :type instance_monitoring: bool
         :param instance_monitoring: Whether instances in group are launched
-                                    with detailed monitoring.
+            with detailed monitoring.
+
+        :type spot_price: float
+        :param spot_price: The spot price you are bidding.  Only applies
+            if you are building an autoscaling group with spot instances.
+
+        :type instance_profile_name: string
+        :param instance_profile_name: The name or the Amazon Resource
+            Name (ARN) of the instance profile associated with the IAM
+            role for the instance.
+
+        :type ebs_optimized: bool
+        :param ebs_optimized: Specifies whether the instance is optimized
+            for EBS I/O (true) or not (false).
         """
         self.connection = connection
         self.name = name
@@ -141,7 +159,10 @@ class LaunchConfiguration(object):
         self.user_data = user_data
         self.created_time = None
         self.instance_monitoring = instance_monitoring
+        self.spot_price = spot_price
+        self.instance_profile_name = instance_profile_name
         self.launch_configuration_arn = None
+        self.ebs_optimized = ebs_optimized
 
     def __repr__(self):
         return 'LaunchConfiguration:%s' % self.name
@@ -150,7 +171,8 @@ class LaunchConfiguration(object):
         if name == 'SecurityGroups':
             return self.security_groups
         elif name == 'BlockDeviceMappings':
-            self.block_device_mappings = ResultSet([('member', BlockDeviceMapping)])
+            self.block_device_mappings = ResultSet([('member',
+                                                     BlockDeviceMapping)])
             return self.block_device_mappings
         elif name == 'InstanceMonitoring':
             self.instance_monitoring = InstanceMonitoring(self)
@@ -166,21 +188,29 @@ class LaunchConfiguration(object):
         elif name == 'ImageId':
             self.image_id = value
         elif name == 'CreatedTime':
-            self.created_time = datetime.strptime(value, '%Y-%m-%dT%H:%M:%S.%fZ')
+            self.created_time = boto.utils.parse_ts(value)
         elif name == 'KernelId':
             self.kernel_id = value
         elif name == 'RamdiskId':
             self.ramdisk_id = value
         elif name == 'UserData':
-            self.user_data = base64.b64decode(value)
+            try:
+                self.user_data = base64.b64decode(value)
+            except TypeError:
+                self.user_data = value
         elif name == 'LaunchConfigurationARN':
             self.launch_configuration_arn = value
         elif name == 'InstanceMonitoring':
             self.instance_monitoring = value
+        elif name == 'SpotPrice':
+            self.spot_price = float(value)
+        elif name == 'IamInstanceProfile':
+            self.instance_profile_name = value
+        elif name == 'EbsOptimized':
+            self.ebs_optimized = True if value.lower() == 'true' else False
         else:
             setattr(self, name, value)
 
     def delete(self):
         """ Delete this launch configuration. """
         return self.connection.delete_launch_configuration(self.name)
-
